/*
 * Sorter
 * This function is supposed to find what sort the user has selected,
 * and change the order of the table of items according to their selection.
 * Paramaters - None
 * Inputs: <select> element with 3 options:
 *    Alphabetically (1-9,A-Z)
 *    Difficulty
 *    Rating
 * Return - None
 * Outputs: <tr> row element changes of innerHTML
 * The sorter function has an embedded function with 3 different forms:
 *    sortKind
 *    This function is used by the JavaScript built-in sort() function.
 *    Paramaters - t1, t2
 *      t1 - First item to be compared from array of row pairs
 *      t2 - Second item to be compared from array of row pairs
 *    Return - Integer of value of either -1(less than), 0(equal),
 *       or 1(greater than)
 *    Form 1: Alphabetical sort - Sorts by title of each item
 *    Form 2: Difficulty sort - Sorts by difficulty given in description
 *    Form 3: Rating sort - Sorts by 'My Rating' given in description
 *
 *
 *  Date of Creation: January 3rd, 2017
 *  Date of Completion: January 12th, 2017
*/
function sorter() {
  //Holds the index of the option selected from the <select> element
  var selected;
  selected = document.getElementById("sort").selectedIndex;
  //To hold the function to be used to sort the table
  var sortKind;
  //Switch statement depending on option selected
  switch (document.getElementById("sort").children[selected].value) {
    //'title' is the alphabetical order sort option
    case "title":
      //Set the sortKind as a custom function
      sortKind = function (t1, t2) {
        //Hold the 2 titles of the 2 items selected
        var n1, n2;
        n1 = t1[0].getElementsByTagName("TH")[0].innerHTML;
        n2 = t2[0].getElementsByTagName("TH")[0].innerHTML;
        /*The idea for the type of return below was from an example made by
        Sharanya Dutta here:
        http://codereview.stackexchange.com/questions/37632/sorting-an-html-table-with-javascript
        */
        return (n1 == n2) ? 0 : ((n1 > n2) ? 1 : -1);
      };
      break;

    //'difficulty' is the puzzle/special pattern difficulty sort option
    case "difficulty":
      sortKind = function (t1, t2) {
        //An array to hold the 2 items to be compared
          //so they can be looped through
        var t;
        t = [t1, t2];
        //To hold the difficulty ratings
        var dif;
        dif = [-1, -1];
        //Loops through each item, to grab the difficulty rating from each
        for(var ii = 0; ii < 2; ii++) {
          //To hold the text to be disected
          var n;
          n = t[ii];
          //Get the inner html
          n = n[1].getElementsByTagName("TH")[1].innerHTML;
          //Clip off back end of text
          n = n.slice(n.indexOf("Difficulty:") + 11);
          //Clip off front end of text (<br>, since DOM changes <br /> to it)
          n = n.slice(0, n.indexOf("<br>"));
          //Trim whitespace and make uniform by making all lowercase
          n = n.trim().toLowerCase();
          //Rank according to difficulty gradings:
          var ts; //Temporary storage
          switch(n) {
            case "none":
              ts = 0;
              break;
            case "very easy":
              ts = 1;
              break;
            case "easy":
              ts = 2;
              break;
            case "medium easy":
              ts = 3;
              break;
            case "medium":
              ts = 4;
              break;
            case "medium hard":
              ts = 5;
              break;
            case "hard":
              ts = 6;
              break;
            case "very hard":
              ts = 7;
              break;
            case "extreme":
              ts = 8;
              break;
            case "almost insolvable":
              ts = 9;
              break;
            default:
              ts = -1;
              break;
          }
          dif[ii] = ts; //Record the difficulty grading
        }
        return (dif[0] == dif[1]) ? 0 : ((dif[0] > dif[1]) ? 1 : -1);
      };
      break;

    //'rating' is the puzzle rating sort option
    case "rating":
      sortKind = function (t1, t2) {
        //An array to hold the 2 items to be compared
          //so they can be looped through
        var t;
        t = [t1, t2];
        //To hold the personal ratings
        var rating;
        rating = [-1, -1];
        //Loops through each item, to grab the difficulty rating from each
        for(var ii = 0; ii < 2; ii++) {
          //To hold the text to be disected
          var n;
          n = t[ii];
          //Get the inner html
          n = n[1].getElementsByTagName("TH")[1].innerHTML;
          //Clip off back end of text
          n = n.slice(n.indexOf("My Rating:") + 10);
          //Clip off front end of text (slash, so that the '/10' is discarded)
          n = n.slice(0,n.indexOf("/"));
          //Trim whitespace
          n = n.trim()
          //Convert into integer and save as rating:
          rating[ii] = Number(n);
        }
        return (rating[0] == rating[1]) ? 0 : ((rating[0] > rating[1]) ? 1 : -1);
      };
        break;
  }
  //Holds the number of row elements in the table
  var trcount;
  trcount = document.getElementsByTagName("TR").length;
  //To hold the references to each of the items in the table
  var trhold;
  trhold = new Array(trcount/2); //Create an array of size equal to # of items
  //Go through each item in array, and add 2 slots for the references to the
    //2 rows
  for(var i = 0; i < trcount / 2; i++) {
    trhold[i] = new Array(2);
  }
  //Go through the table, and place each set of 2 rows into the trhold array
  for(var i = 0; i < trcount; i += 2) {
    trhold[i / 2][0] = document.getElementsByTagName("TR")[i];
    trhold[i / 2][1] = document.getElementsByTagName("TR")[i + 1];
  }
  //Sort the array using the custom sortKind function
  trhold.sort(sortKind);

  //To hold the new innerHTML of the rows in the new order to be written to the
    //table
  var newversion;
  newversion = [trcount];
  //Loop through, copying the html from the table rows into the newversion array
    //in the order they were sorted inside the trhold array
  for(var i = 0; i < trcount; i += 2) {
    newversion[i] = trhold[i / 2][0].innerHTML;
    newversion[i + 1] = trhold[i / 2][1].innerHTML;
  }
  //Overwrite the table with the newversion
  for(var i = 0; i < trcount; i++) {
    document.getElementsByTagName("TR")[i].innerHTML = newversion[i];
  }
  filter();//Refilter with the new organization
}

/*
 * Filter
 * This function is supposed to find what categories are selected in the nav,
 * and hide or show items according to the categories selected.
 * Paramaters - None
 * Inputs: <form> element with 4 checkbox items:
 *    Cuboid
 *    Shape Shifting
 *    Rubik's Cube Design
 *    Other
 * Return - None
 * Outputs: <tr> row element display property changes
 *
 *
 *  Date of Creation: January 10th, 2017
 *  Date of Completion: January 14th, 2017
*/
function filter() {
  //Holds the checked/unchecked status of the Cuboid checkbox
  var sCuboid;
  sCuboid = true;
  if (document.getElementById("cuboid").checked == false) {
    sCuboid = false;
  }
  //Holds the checked/unchecked status of the Shape Shifting checkbox
  var sShape;
  sShape = true;
  if (document.getElementById("shape").checked == false) {
    sShape = false;
  }
  //Holds the checked/unchecked status of the Rubik's Cube Design checkbox
  var sDesign;
  sDesign = true;
  if (document.getElementById("design").checked == false) {
    sDesign = false;
  }
  //Holds the checked/unchecked status of the Other checkbox
  var sOther;
  sOther = true;
  if (document.getElementById("other").checked == false) {
    sOther = false;
  }

  //Holds the number of row elements in the table
  var trcount;
  trcount = document.getElementsByTagName("TR").length;
  //To hold the references to each of the items in the table (as in sorter())
  var trhold;
  trhold = new Array(trcount/2);
  for(var i = 0; i < trcount / 2; i++) {
    trhold[i] = new Array(2);
  }
  for(var i = 0; i < trcount; i += 2) {
    trhold[i / 2][0] = document.getElementsByTagName("TR")[i];
    trhold[i / 2][1] = document.getElementsByTagName("TR")[i + 1];
  }

  //Loop through each item in the array
  for(var i = 0; i < trcount; i += 2) {
      //To hold the text to be disected
      var n;
      n = trhold[i / 2];
      //Get the inner html
      n = n[1].getElementsByTagName("TH")[1].innerHTML;
      //Clip off back end of text
      n = n.slice(n.indexOf("Category:") + 9);
      //Clip off front end of text
      n = n.slice(0, n.indexOf("<br>"));
      //Trim whitespace and make uniform with all lowercase
      n = n.trim().toLowerCase();

      //Switch according to the category of the item
        //If checkbox is checked if(categoryCheck)
          //Display the rows as normal
        //Else
          //Hide the rows by changing display to none
      switch(n) {
        case "cuboid":
          if (sCuboid) {
            trhold[i / 2][0].style.display = "table-row";
            trhold[i / 2][1].style.display = "table-row";
          } else {
            trhold[i / 2][0].style.display = "none";
            trhold[i / 2][1].style.display = "none";
          }
          break;

        case "shape shifting":
          if (sShape) {
            trhold[i / 2][0].style.display = "table-row";
            trhold[i / 2][1].style.display = "table-row";
          } else {
            trhold[i / 2][0].style.display = "none";
            trhold[i / 2][1].style.display = "none";
          }
          break;

        case "rubik design":
          if (sDesign) {
            trhold[i / 2][0].style.display = "table-row";
            trhold[i / 2][1].style.display = "table-row";
          } else {
            trhold[i / 2][0].style.display = "none";
            trhold[i / 2][1].style.display = "none";
          }
          break;

        case "other":
          if (sOther) {
            trhold[i / 2][0].style.display = "table-row";
            trhold[i / 2][1].style.display = "table-row";
          } else {
            trhold[i / 2][0].style.display = "none";
            trhold[i / 2][1].style.display = "none";
          }
          break;
      }
  }
}
